@@ -28,6 +28,10 @@
 #include <fcntl.h>
 #include <ctype.h>
 
+#if defined(__sun)
+typedef uint32_t u_int32_t;
+#endif
+
 #include "dvdauthor.h"
 #include "da-internal.h"
 #include "libauthor.h"
@@ -980,11 +984,11 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
 // fprintf(stderr,"NewScr: %lld BackOffs: %lld\n", newscr, backoffs );
                 if((newscr==0)&&(lastscr>0))
                 {
-                  fprintf(stderr,"\nWARN:  SCR reset. backoffs=%lld  lastscr=%lld\n",backoffs,lastscr);
+                  fprintf(stderr,"\nWARN:  SCR reset. backoffs=%lld  lastscr=%lld\n",(long long int)backoffs,(long long int)lastscr);
                   backoffs-=lastscr;
-                  fprintf(stderr,"\nWARN:  SCR reset. New back offset = %lld \n",backoffs);
+                  fprintf(stderr,"\nWARN:  SCR reset. New back offset = %lld \n",(long long int)backoffs);
                 } else if( newscr < lastscr ) {
-                    fprintf(stderr,"ERR:  SCR moves backwards, remultiplex input. (%lld < %lld)\n", newscr, lastscr );
+                    fprintf(stderr,"ERR:  SCR moves backwards, remultiplex input. (%lld < %lld)\n", (long long int)newscr, (long long int)lastscr );
 //                    Exit(1);
                 }
                 lastscr=newscr;
