@@ -55,23 +55,8 @@ COMMENTAIRE :
 #include "ac.h"
 #include "dvdcell.h"
 #include "../dvdauthor/libauthor.h"
-
-#define MAX_VOB_SIZE		(1024*1024*1024)
-
-typedef struct
-{
-	int etat;
-	dvd_file_t * file_handle;
-	uint32_t sector;
-	uint32_t dsi_next_vobu;
-	Cell_t * cell;
-	char * targetDir;
-	unsigned char * buf;
-	int bufASize;
-	unsigned char * bufPtr;
-	int bufSize;
-} VapContext_t;
-
+#include "vapcontext.h"
+#include "mpeg2dec.h"
 
 static char TitreDvd[128];
 
@@ -92,8 +77,8 @@ static uint32_t Position;
 static unsigned char TempBuf[DVD_VIDEO_LB_LEN * 2];
 static int TempSize = 0;
 
-static	double AuthorlPct = -1;
-static	int AuthorFixStage = 0;
+static double AuthorlPct = -1;
+static int AuthorFixStage = 0;
 static long long AuthorSize;
 
 /*@$#[dvd2vob.c] static proto. AutoProtoSigV1.1. date: 106/02/16 21:33:23 */
@@ -189,6 +174,9 @@ char * targetDir;
 		cBuff = TempBuf;
 	}
 	if( TempSize >= DVD_VIDEO_LB_LEN ) {
+		if( !isNavPack( cBuff ) ) {
+			Mpeg2Demux( cBuff, DVD_VIDEO_LB_LEN );
+		}
 		if( fwrite( cBuff, 1, DVD_VIDEO_LB_LEN, OutFp ) != DVD_VIDEO_LB_LEN )	return( -1 );
 		OutSize += DVD_VIDEO_LB_LEN;
 		TotalSize += DVD_VIDEO_LB_LEN;
@@ -244,6 +232,35 @@ static void MakeRoom( VapContext_t * c, int size )
 	}
 }
 
+/*------------------------------------------------------------------------------
+	_FINDNEXTVOBU-
+Linux!jef 2007/08/16 01:37:18
+------------------------------------------------------------------------------*/
+
+static uint32_t _FindNextVobu( c )
+VapContext_t * c;
+{
+	vobu_admap_t * vobu_admap;
+	uint32_t length, i;
+
+	vobu_admap = c->cell->ifo->vts_vobu_admap;
+	length = vobu_admap->last_byte + 1 - VOBU_ADMAP_SIZE;
+
+	for( i = 0; i < length/sizeof(uint32_t); i++) {
+// DBG('b',fprintf(stderr,"%s: vobu at sector: %d\n", __FUNCTION__, vobu_admap->vobu_start_sectors[i] ););
+/* Vobu adress map seems to be ordered */
+		if( vobu_admap->vobu_start_sectors[i] > c->sector) {
+			DBG('b',fprintf(stderr,"%s: returning next vobu at sector: %d\n", __FUNCTION__, vobu_admap->vobu_start_sectors[i] ););
+			return( vobu_admap->vobu_start_sectors[i] );
+		}
+//		if( vobu_admap->vobu_start_sectors[i] == c->sector) {
+//			return( vobu_admap->vobu_start_sectors[i+1] );
+//		}
+	}
+	DBG('b',fprintf(stderr,"%s: nextvobu notfound for sector: %d\n", __FUNCTION__, c->sector ););
+	return( SRI_END_OF_CELL );
+}
+
 /*------------------------------------------------------------------------------
 	READIN-
 Linux!jef 2006/01/24 22:28:46
@@ -256,6 +273,7 @@ static int Readin( void * parm, unsigned char * buffer, int size )
 	int len;
 	dsi_t	dsi_pack;
 	uint32_t nsectors;
+	int badNavPack = 0;
 
 //	DBG('b',fprintf(stderr,"%s: size: %d inBuf: %d\n", __FUNCTION__, size, c->bufSize ););
 
@@ -283,8 +301,16 @@ static int Readin( void * parm, unsigned char * buffer, int size )
 
 /* read nav pack */
 	MakeRoom( c, DVD_VIDEO_LB_LEN );
-	len = MyDVDReadBlocks( c->file_handle, c->sector, 1, c->bufPtr );
-	if( len != 1 )	return( -1 );
+	len = MyDVDRead1Block( c->file_handle, c->sector, c->bufPtr );
+	if( len == -1 )	return( -1 );
+/* parse contained DSI pack */
+	navRead_DSI( &dsi_pack, c->bufPtr + DSI_START_BYTE );
+	if( len == 0 || dsi_pack.dsi_gi.nv_pck_lbn != c->sector ) {
+		CreateDummyNavPack( c->bufPtr, c->sector );
+		badNavPack = 1;
+		c->dsi_next_vobu = _FindNextVobu( c );
+	}
+
 	len = DVD_VIDEO_LB_LEN;
 	c->bufSize += len;
 
@@ -296,22 +322,49 @@ static int Readin( void * parm, unsigned char * buffer, int size )
 		CurCell->oldLastSector = c->sector;
 		v = NewVobu( &CurCell->va, c->sector );
 	}
-/* parse contained DSI pack */
-	navRead_DSI( &dsi_pack, c->bufPtr + DSI_START_BYTE );
-	nsectors      = dsi_pack.dsi_gi.vobu_ea;
-	c->dsi_next_vobu = dsi_pack.vobu_sri.next_vobu;
 
-	MakeRoom( c, nsectors*DVD_VIDEO_LB_LEN );
+	if( badNavPack ) {
+		MakeRoom( c, 1 * DVD_VIDEO_LB_LEN );
+		CreateDummyPack( c->bufPtr + DVD_VIDEO_LB_LEN );
+		nsectors = 1;
+	}
+	else {
+		int i;
+		unsigned char * cBuff;
+
+		nsectors  = dsi_pack.dsi_gi.vobu_ea;
+		if( !nsectors ) { /* Protected dvd ? */
+			DBG('b',fprintf(stderr,"%s: no sectors in DSI !\n", __FUNCTION__););
+			return( 0 ); /* EOF */
+		}
+		c->dsi_next_vobu = dsi_pack.vobu_sri.next_vobu;
+
+		MakeRoom( c, nsectors*DVD_VIDEO_LB_LEN );
+		cBuff = c->bufPtr + DVD_VIDEO_LB_LEN;
 /* read VOBU */
-	len = MyDVDReadBlocks( c->file_handle, (c->sector + 1), nsectors, c->bufPtr + DVD_VIDEO_LB_LEN );
-	if( len != nsectors )	return( -1 );
+		for( i = 0; i < nsectors; i++ ) {
+			len = MyDVDRead1Block( c->file_handle, c->sector + 1 + i, cBuff );
+// fprintf(stderr,"PACK: MyDVDRead1Block(%d)=%d\n", c->sector+1+i, len );
+			if( len == -1 )	return( -1 );
+			if( len == 0 ) {
+				CreateDummyPack( cBuff );
+				nsectors = 1;
+				break;
+			}
+			cBuff += DVD_VIDEO_LB_LEN;
+		}
+	}
 	len = nsectors * DVD_VIDEO_LB_LEN;
 	c->bufSize += len;
 
 	Avancement( len );
 	if( AvancementUi( NULL ) < 0 )	return( -1 );
-
-	c->sector += (c->dsi_next_vobu & 0x7fffffff);
+	if( badNavPack ) {
+		c->sector = c->dsi_next_vobu;
+		c->dsi_next_vobu  = 0;
+	}
+	else
+		c->sector += (c->dsi_next_vobu & 0x7fffffff);
 
 	return( readed + Readin( parm, buffer, size ) );
 }
@@ -715,30 +768,30 @@ static double ReCpuFactor( )
 	tSize += (long long)vNavSize * DVD_BLOCK_LEN;
 
 
-	DBG( 'b',fprintf(stderr,"0) VAPORIZE VIDEO SIZE: %lld\n", (long long)vVideoSize * DVD_BLOCK_LEN ););
-	DBG( 'b',fprintf(stderr,"0) VAPORIZE AUDIO SIZE: %lld\n", (long long)vAudioSize * DVD_BLOCK_LEN ););
-	DBG( 'b',fprintf(stderr,"0) VAPORIZE SUBP  SIZE: %lld\n", (long long)vSubpSize * DVD_BLOCK_LEN ););
-	DBG( 'b',fprintf(stderr,"0) VAPORIZE NAV   SIZE: %lld\n", (long long)vNavSize * DVD_BLOCK_LEN ););
+	DBG( 'b',fprintf(stderr,"0) VAPORIZE VIDEO SIZE: %s\n", LLSize2Giga((long long)vVideoSize * DVD_BLOCK_LEN )););
+	DBG( 'b',fprintf(stderr,"0) VAPORIZE AUDIO SIZE: %s\n", LLSize2Giga((long long)vAudioSize * DVD_BLOCK_LEN )););
+	DBG( 'b',fprintf(stderr,"0) VAPORIZE SUBP  SIZE: %s\n", LLSize2Giga((long long)vSubpSize * DVD_BLOCK_LEN )););
+	DBG( 'b',fprintf(stderr,"0) VAPORIZE NAV   SIZE: %s\n", LLSize2Giga((long long)vNavSize * DVD_BLOCK_LEN )););
 
 
-	DBG( 'b',fprintf(stderr,"THEORICAL TOTAL SIZE: %lld\n", tSize ););
-	DBG( 'b',fprintf(stderr,"PRATICAL TOTAL SIZE : %lld\n", TotalSize ););
+	DBG( 'b',fprintf(stderr,"THEORICAL TOTAL SIZE: %s\n", LLSize2Giga(tSize)););
+	DBG( 'b',fprintf(stderr,"PRATICAL TOTAL SIZE : %s\n", LLSize2Giga(TotalSize)););
 
 	VaporizePacks( &vVideoSize, NULL );
 
-	DBG( 'b',fprintf(stderr,"VAPORIZE VIDEO SIZE: %lld\n", (long long)vVideoSize * DVD_BLOCK_LEN ););
-	DBG( 'b',fprintf(stderr,"VAPORIZE AUDIO SIZE: %lld\n", (long long)vAudioSize * DVD_BLOCK_LEN ););
-	DBG( 'b',fprintf(stderr,"VAPORIZE SUBP  SIZE: %lld\n", (long long)vSubpSize * DVD_BLOCK_LEN ););
-	DBG( 'b',fprintf(stderr,"VAPORIZE NAV   SIZE: %lld\n", (long long)vNavSize * DVD_BLOCK_LEN ););
+	DBG( 'b',fprintf(stderr,"VAPORIZE VIDEO SIZE: %s\n", LLSize2Giga((long long)vVideoSize * DVD_BLOCK_LEN )););
+	DBG( 'b',fprintf(stderr,"VAPORIZE AUDIO SIZE: %s\n", LLSize2Giga((long long)vAudioSize * DVD_BLOCK_LEN )););
+	DBG( 'b',fprintf(stderr,"VAPORIZE SUBP  SIZE: %s\n", LLSize2Giga((long long)vSubpSize * DVD_BLOCK_LEN )););
+	DBG( 'b',fprintf(stderr,"VAPORIZE NAV   SIZE: %s\n", LLSize2Giga((long long)vNavSize * DVD_BLOCK_LEN )););
 
 	videoSize = ( (long long)vVideoSize * (long long)DVD_BLOCK_LEN * (long long)totalSectors ) / (long long)doneSectors;
-	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL VIDEOSIZE: %lld\n", videoSize ););
+	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL VIDEOSIZE: %s\n", LLSize2Giga(videoSize)););
 	audioSize = ( (long long)vAudioSize * (long long)DVD_BLOCK_LEN * (long long)totalSectors ) / (long long)doneSectors;
-	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL AUDIOSIZE: %lld\n", audioSize ););
+	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL AUDIOSIZE: %s\n", LLSize2Giga(audioSize)););
 	subpSize = ( (long long)vSubpSize * (long long)DVD_BLOCK_LEN * (long long)totalSectors ) / (long long)doneSectors;
-	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL SUBSIZE: %lld\n", subpSize ););
+	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL SUBSIZE: %s\n", LLSize2Giga(subpSize)););
 	navSize = ( (long long)vNavSize * (long long)DVD_BLOCK_LEN * (long long)totalSectors ) / (long long)doneSectors;
-	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL NAVSIZE: %lld\n", navSize ););
+	DBG( 'b',fprintf(stderr,"2) ESTIMATED TOTAL NAVSIZE: %s\n", LLSize2Giga(navSize)););
 
 /* On calcul ce qu'il reste a faire */
 	remainSectors = totalSectors - doneSectors;
@@ -748,11 +801,11 @@ static double ReCpuFactor( )
 	remainNavSize = navSize - ((long long)vNavSize * (long long)DVD_BLOCK_LEN);
 	remainSize = GetWantedBytes() - TotalSize;
 
-	DBG( 'b',fprintf(stderr,"3) REMAIN VIDEOSIZE: %lld\n", remainVideoSize ););
-	DBG( 'b',fprintf(stderr,"3) REMAIN AUDIOSIZE: %lld\n", remainAudioSize ););
-	DBG( 'b',fprintf(stderr,"3) REMAIN   SUBSIZE: %lld\n", remainSubpSize ););
-	DBG( 'b',fprintf(stderr,"3) REMAIN   NAVSIZE: %lld\n", remainNavSize ););
-	DBG( 'b',fprintf(stderr,"3) REMAIN   DVDSIZE: %lld\n", remainSize ););
+	DBG( 'b',fprintf(stderr,"3) REMAIN VIDEOSIZE: %s\n", LLSize2Giga(remainVideoSize)););
+	DBG( 'b',fprintf(stderr,"3) REMAIN AUDIOSIZE: %s\n", LLSize2Giga(remainAudioSize)););
+	DBG( 'b',fprintf(stderr,"3) REMAIN   SUBSIZE: %s\n", LLSize2Giga(remainSubpSize)););
+	DBG( 'b',fprintf(stderr,"3) REMAIN   NAVSIZE: %s\n", LLSize2Giga(remainNavSize)););
+	DBG( 'b',fprintf(stderr,"3) REMAIN   DVDSIZE: %s\n", LLSize2Giga(remainSize)););
 
 /* Les packets de navigations sont de la video non compressible ! */
 
@@ -806,6 +859,7 @@ int CopyDvd( GtkWidget * forme, int noPisteVideo, AudioMap_t * audioMap, SubMap_
 	if( ConfigGetInt( KEY_MKISO, 0 ) ) {
 		fsize = FsFree( isoDir ) - GetWantedBytes();
 		if( fsize < 0 ) {
+fprintf(stderr,"FsFree(%lld) GetWantedBytes(%lld)\n", FsFree( isoDir ), GetWantedBytes() );
 			PlayError();
 			MessageBoxError( _("Pas assez de place sur %s\nIl manque %lld octets !"), isoDir, -fsize );
 			return( -1 );
@@ -893,6 +947,8 @@ int CopyDvd( GtkWidget * forme, int noPisteVideo, AudioMap_t * audioMap, SubMap_
 		dynQual = 1;
 	}
 
+	Mpeg2Init();
+
 	VaporizeFactor( factor, dynQual );
 	VaporizeInjection( injectionFile );
 
@@ -951,6 +1007,8 @@ out:;
 
 	if( OutFp ) { fclose( OutFp ); OutFp = NULL; }
 
+	Mpeg2End();
+
 #if TEST_AUTHOR == 2
 	goto out2;
 #endif
@@ -960,22 +1018,37 @@ author:;
 #endif
 	if( !ret ) { /* DvdAuthor */
 		char * args[20];
+		char * tmp;
 		int argc = 0;
 
 		DisplayAvancement( _("Création de la structure du DVD.\n") );
 		chapitreStr = BuildChapitreString( noPisteVideo );
 		args[argc++] = strdup( "dvdauthor" );
 		args[argc++] = strdup( "-t" );
-		args[argc++] = strdup( "-a" );
-		args[argc++] = BuildAudioString( noPisteVideo, audioMap );
+		tmp = BuildAudioString( noPisteVideo, audioMap );
+		if( tmp ) {
+			if( *tmp ) {
+				args[argc++] = strdup( "-a" );
+				args[argc++] = tmp;
+			}
+			else
+				free( tmp );
+		}
 		args[argc++] = strdup( "-c" );
 		args[argc++] = chapitreStr;
 		args[argc++] = strdup( "-v" );
 		args[argc++] = BuildFormatString( noPisteVideo ); // strdup( "pal+16:9+nopanscan+720x576" );
 		args[argc++] = strdup( "-p" );
 		args[argc++] = strdup( pFile );
-		args[argc++] = strdup( "-s" );
-		args[argc++] = BuildSubString( noPisteVideo, subMap );
+		tmp = BuildSubString( noPisteVideo, subMap );
+		if( tmp ) {
+			if( *tmp ) {
+				args[argc++] = strdup( "-s" );
+				args[argc++] = tmp;
+			}
+			else
+				free( tmp );
+		}
 		args[argc++] = strdup( "-o" );
 		args[argc++] = strdup( targetDir);
 		args[argc++] = strdup( "-f" );
