@@ -320,8 +320,10 @@ int ScanDvd( GtkWidget * forme )
 
 		vtsi_mat = ifo->vtsi_mat;
 		vts_pgcit = ifo->vts_pgcit;
+/* HUM HUM */
+//		piste->vts = j+1;
+		piste->vts = Ifo_zero->tt_srpt->title[j].title_set_nr;
 
-		piste->vts = j+1;
 		piste->ifo = ifo;
 		piste->vtsi_mat = vtsi_mat;
 		piste->vts_pgcit = vts_pgcit;
@@ -942,7 +944,7 @@ int ScanPiste( GtkWidget * forme, int noPisteVideo )
 			audio_attr = &vtsi_mat->vts_audio_attr[k];
 			sprintf( lang_code, "%c%c", audio_attr->lang_code >> 8,audio_attr->lang_code & 0xff);
 
-			DBG('c',fprintf(stderr,"Audio%d=0x%x %s Id: %d Channels: %d\n",k, pgc->subp_control[k], DvdLangCode2String( lang_code ), AudioId(pgc->audio_control[k]), audio_attr->channels + 1 ););
+			DBG('c',fprintf(stderr,"Audio%d=0x%x %s (0x%x) Id: %d Channels: %d\n",k, pgc->subp_control[k], DvdLangCode2String( lang_code ), audio_attr->lang_code, AudioId(pgc->audio_control[k]), audio_attr->channels + 1 ););
 
 			gtk_tree_store_append(treestore, &iter, NULL );
 			sprintf( string, _("%d"), k+1 );
@@ -1103,7 +1105,8 @@ int ScanPiste( GtkWidget * forme, int noPisteVideo )
 			sprintf( string, _("%s"), DvdLangCode2String( lang_code ) );
 			gtk_tree_store_set( treestore, &iter, SUB_COL3_LANG, string, -1 );
 
-			sprintf( string, _("%s"), SubpType[subp_attr->lang_extension] );
+//			sprintf( string, _("%s"), SubpType[subp_attr->lang_extension] );
+			sprintf( string, _("%s"), SubpType[subp_attr->code_extension] );
 			gtk_tree_store_set( treestore, &iter, SUB_COL4_EXT, string, -1 );
 
 			gtk_tree_store_set( treestore, &iter, SUB_COL5_LANGCODE, lang_code, -1 );
