@@ -14,7 +14,7 @@ COMMENTAIRE :
 
 #ifndef __SYSTOOLS_H__
 #define __SYSTOOLS_H__
-/*@$#[systools.c] global proto. AutoProtoSigV1.1. date: 107/01/25 21:28:57 */
+/*@$#[systools.c] global proto. AutoProtoSigV1.1. date: 107/02/12 20:56:44 */
 #include "proto.h"
 #ifdef __cplusplus
 extern "C" {
@@ -34,7 +34,8 @@ char *AddString2List PROTO((char *list, char *string));
 char *RemoveStringFromList PROTO((char *list, char *string));
 int MakeDirIfNeeded PROTO((char *path));
 int IsDvdLoaded PROTO((char *device));
-int StringDuree2Sec( char * string );
+int StringDuree2Sec PROTO((char *string));
+int IsLoopDeviceReady PROTO((char *loopDevice));
 #ifdef __cplusplus
 }
 #endif
